require 'installer/action'
require 'installer/errors'
require 'net/ssh'

class Installer::Planner::Step::ConnectSSH < Installer::Planner::Step
  def perform_step(&stream_block)
    @infra.hosts.each do |hostname, host|
      if !host.localhost
        if host.ssh_key_file
          begin
            verify_key_file(host.ssh_key_file, host.ssh_key_passphrase)
            info(hostname, "SSH private key file #{host.ssh_key_file} verified for #{hostname}.")
          rescue Installer::Error => e
            error(hostname, "There is a problem with the SSH private key: #{e.message}.")
            next
          end
        end

        begin
          host.ssh_connect
          info(hostname, "SSH connection to #{hostname} established successfully.")
        rescue Installer::RemoteError => e
          error(hostname, "Could not open an SSH connection: #{e.message}.")
        end
      end
    end
  end

  def fail_stop?
    true
  end

  def name
    :connect_ssh
  end

  def verify_key_file(filename, passphrase=nil)
    if filename && !filename.strip.empty?
      unless File.readable?(filename)
        raise(Installer::Error, "'#{filename}' can not be read")
      end

      unless File.size(filename) > 0
        raise(Installer::Error, "'#{filename}' is empty")
      end

      unless /PRIVATE KEY/.match(File.read(filename))
        raise(Installer::Error, "'#{filename}' does not appear to be a private key")
      end

      unless (File.stat(filename).mode & 0077) == 0
        raise(Installer::Error, "'#{filename}' permissions are too open")
      end

      begin
        # In lieu of keyword args, explicit variable for the argument
        ask_passphrase = false
        Net::SSH::KeyFactory.load_private_key(filename, passphrase, ask_passphrase)
      rescue OpenSSL::PKey::PKeyError, ArgumentError => e
        if /Could not parse PKey: no start line/ =~ e.message
          raise(Installer::Error, "'#{filename}' could not be parsed. The passphrase may be incorrect or the private key file may be corrupt")
        else
          raise(Installer::Error, "'#{filename}' had an OpenSSL PKey Error: #{e}")
        end
      end
    end
  end

  def description
    "Verifying that #{@infra.installer_hostname} can SSH to #{@infra.hosts.keys.join(', ')}."
  end
end
